<?php
if (!defined('APS_DEVELOPMENT_MODE')) define('APS_DEVELOPMENT_MODE', 'on');

require_once "aps/2/runtime.php";
require 'contexts.php';

// Definition of type structures

class CPU
{
    /**
     * @type("integer")
     * @title("Number of CPUs")
     * @description("Number of CPU cores")
     */
    public $number;
}

class OS
{
    /**
     * @type("string")
     * @title("OS Name")
     * @description("Operating System Name")
     */
    public $name;

    /**
     * @type("string")
     * @title("OS Version")
     * @description("Operating System version")
     */
    public $version;
}

class Hardware
{
    /**
     * @type("integer")
     * @title("RAM Size")
     * @description("RAM size in GB")
     */
    public $memory;

    /**
     * @type("integer")
     * @title("Disk Space")
     * @description("Disk space in GB")
     */
    public $diskspace;

    /**
     * @type("boolean")
     * @title("VM with separate OS core or Container using shared OS core")
     * @description("VM requires more time for provisioning - async POST")
     */
    public $VM;

    /**
     * @type("CPU")
     * @title("CPU")
     * @description("Server CPU parameters")
     */
    public $CPU;
}

class Platform
{
    /**
     * @type("string")
     * @title("Architecture")
     * @description("Platform architecture")
     */
    public $arch;

    /**
     * @type("OS")
     * @title("OS Parameters")
     * @description("Parameters of operating system")
     */
    public $OS;
}


// Main class
/**
 * @type("http://aps-standard.org/samples/vrd/vps/1.0")
 * @implements("http://aps-standard.org/types/core/resource/1.0")
 */
class vps extends APS\ResourceBase
{

    // Relation with the management context
    /**
     * @link("http://aps-standard.org/samples/vrd/context/1.0")
     * @required
     */
    public $context;

    // VPS properties

    /**
     * @type("string")
     * @title("name")
     * @description("Server Name")
     */
    public $name;

    /**
     * @type("string")
     * @title("Description")
     * @description("Server Description")
     */
    public $description;

    /**
     * @type("string")
     * @title("state")
     * @description("Server State")
     */
    public $state;

    // VPS complex properties (structures) - defined as classes above

    /**
     * @type("Hardware")
     * @title("Hardware")
     * @description("Server Hardware")
     */
    public $hardware;

    /**
     * @type("Platform")
     * @title("Platform")
     * @description("OS Platform")
     */
    public $platform;

    /**
     * @type("integer")
     * @title("Counter for async operation")
     * @description("Counts down number of retries for async operation")
     */
    public $retry;

    // Add Notification ID as a VPS property
    /**
     * @type(string)
     * @title("Notification ID")
     */
    public $notificationId;

    // Standard CRUD operations

    // Provision method is called when a sync provisioning is requested
    public function provision()
    {

        $accountId = $this->context->account->aps->id;

        $vpsFile = fopen($accountId."/".$this->aps->id.".json", 'w');
        fwrite($vpsFile, $this);
        fclose($vpsFile);
    }

    public function unprovision()
    {

        $accountId = $this->context->account->aps->id;

        ## Remove the VPS file with resource parameters:
        unlink($accountId."/".$this->aps->id.".json");

    }

    public function configure($new)
    {
        $accountId = $this->context->account->aps->id;
        file_put_contents($accountId."/".$this->aps->id.".json", $this);
        parent::configure($new);
    }

    // Custom functions called on pressing start/stop buttons in custom UI:
    /**
     * @verb(GET)
     * @path("/start")
     * @return(string,text/json)
     */
    public function start()
    {
        $accountId = $this->context->account->aps->id;
        $this->state = 'Running';
        $apsc = \APS\Request::getController();
        $apsc->updateResource($this);
        file_put_contents($accountId."/".$this->aps->id.".json", $this);
    }

    /**
     * @verb(GET)
     * @path("/stop")
     * @return(string,text/json)
     */
    public function stop()
    {
        $accountId = $this->context->account->aps->id;
        $this->state = 'Stopped';
        $apsc = \APS\Request::getController();
        $apsc->updateResource($this);
        file_put_contents($accountId."/".$this->aps->id.".json", $this);
    }
}
