define([
	"dojo/_base/declare",
	"dojo/Deferred",
	"dojo/Stateful",
	"dojox/mvc/at",

	"aps/ResourceStore",
	"aps/Tile",
	"aps/Output",
	"aps/common",
	"aps/nav/ViewPlugin"
], function (
	declare,
	Deferred,
	Stateful,
	at,

	Store,
	Tile,
	Output,
	common,
	ViewPlugin
) {
	return declare(ViewPlugin, {
		init: function (mediator) {
			var self = this;
			var vpsCount = new Stateful({
				usage: 0,
				limit: 0
			});
			var widget = new Tile({
				id: this.genId("vpsList"),
				title: _("VPSes", self),
				onClick: function () {
					aps.apsc.gotoView("http://aps-standard.org/samples/vrd#servers");
				},
				buttons: [{
					title: _("Create new VPS", self),
					iconClass: "fa-plus",
					autoBusy: false,
					type: "link",
					onClick: function () {
						aps.apsc.gotoView("http://aps-standard.org/samples/vrd#server-new-1");
					}
				}]
			});

			mediator.getWidget = function () {
				return widget;
			};

			mediator.watch("resourceUsage", function (name, oldvalue, newvalue) {
				if (newvalue !== undefined) {
					common.forEach(newvalue, function(subscriptionResources) { // All resources of a subscription
						subscriptionResources.forEach(function(resource) {
							if(resource.apsType === "http://aps-standard.org/samples/vrd/vps/1.0") {
								vpsCount.set({
									"usage": resource.usage,
									"limit": resource.limit
								});
							}
						});
					});
				} else {
					vpsCount.set({
						"usage": 0,
						"limit": 0
					});
				}

				var output = self.byId("dash_vps_plugin_out");

				if (!output) {
					output = new Output({
						id: this.genId("dash_vps_plugin_out"),
						value: at(vpsCount, "usage"),
						limit: at(vpsCount, "limit"),
						content: _(
							"Total: ${value} servers out of ${limit}",
							self
						)
					});
					widget.addChild(output);
				} else {
					output.set({
						content: _(
							"Total: ${value} servers out of ${limit}",
							self
						)
					});
				}
			});
		}
	});
});
