define([
    "dojo/_base/declare",
    "dojo/when",
    "dojox/mvc/getStateful",
    "dojox/mvc/at",
    "dojo/_base/array",

    "aps/Memory",
    "aps/View",
    "aps/Status",
    "aps/ResourceStore",
    "aps/xhr",

    "./displayError"
], function (
    declare,
    when,
    getStateful,
    at,
    arr,

    Memory,
    View,
    Status,
    Store,
    xhr,

    displayError
) {
    function getCurrentTime() {
        var d = new Date();
        return(
            d.getUTCFullYear().toString() + '-' +
            (d.getUTCMonth() > 8 ? (d.getUTCMonth()+1).toString() : '0' + (d.getUTCMonth()+1).toString()) + '-' +
            (d.getUTCDate() > 9 ? d.getUTCDate().toString() : '0'+d.getUTCDate().toString()) + 'T' +
            (d.getUTCHours() > 9 ? d.getUTCHours().toString() : '0'+d.getUTCHours().toString()) + ':' +
            (d.getUTCMinutes() > 9 ? d.getUTCMinutes().toString() : '0' + d.getUTCMinutes().toString()) + ':' +
            (d.getUTCSeconds() > 9 ? d.getUTCSeconds().toString() : '0' + d.getUTCSeconds().toString())
        );
    }

    var self, page, vpsGrid, ratedDataGrid, selectionArray, contextId, accountId, parentId, lastPollTime;
    return declare(View, {
        init: function() {
            self = this;

            /* Create the data store */
            var vpsStore = new Store({
                apsType: "http://aps-standard.org/samples/vrd/vps/1.0",
                target: "/aps/2/resources/"
            });

            /* Initialize a cache to collect reports on resource charges */
            var chargeReportCache = new Memory({
                data: [],
                idProperty: "lastInvoiceDate"
            });

            /* Handler for the *New* button */
            var add = function() {
                /* Start the process of creating a VPS by going to the first screen */
                aps.apsc.gotoView("vps-wizard");
            };

            var refresh = function() {
                    vpsGrid.refresh();
                    ratedDataGrid.refresh();
            };


            var pollCharges = function() {
                var currentTime = getCurrentTime();
                xhr.get("/aps/2/resources/" + contextId + "/exportRatedData?lastInvoiceDate=" + lastPollTime)
                .then(function(report) {
                    if(report == null) return;
                    chargeReportCache.add(report);
                    ratedDataGrid.refresh();
                });
                lastPollTime = currentTime;
            };

            //_FUNCTIONS_NOT_MODIFIED_START
            function changeState(state, btn) {
                var counter = selectionArray.length;

                /* Clear the current messages on the screen */
                page.get("messageList").removeAll();

                arr.forEach(selectionArray, function(vpsId){
                    console.log(
                        "I'm trying to change state of VPS with id = [" + vpsId + "]"
                    );

                    /* Save the VPS properties */
                    when(xhr(
                            "/aps/2/resources/" + vpsId + "/" + state,
                            { method: "GET",  handleAs: "text" }
                        ),
                        /* If success, process the next VPS until the list is empty */
                        function(){
                            console.log("State of VPS with id = [" + vpsId + "] changed");
                            selectionArray.splice(selectionArray.indexOf(vpsId),1);
                            vpsGrid.refresh();
                            if (--counter === 0) { btn.cancel(); } /* Make the button available */
                        },
                        /* If failure, call the error handler */
                        function(e){
                            displayError(e);
                            if (--counter === 0) { btn.cancel(); }
                        }
                    );
                });
            }

            /* Create the on-click handler for the *Stop* button */
            var stop = function() {
                changeState("stop", this);
            };

            /* Create the on-click handler for the *Start* button */
            var start = function() {
                changeState("start", this);
            };

            /* Create the on-click handler for the *Delete* button */
            var remove = function() {
                var btn = this;
                /* Get confirmation from the user for the delete operation */
                if (!confirm("Are you sure you want delete VPSes?")) {
                    btn.cancel();
                    return;
                }

                var counter = selectionArray.length;

                /* Clear the current messages on the screen */
                page.get("messageList").removeAll();

                selectionArray.forEach(function(vpsId){
                    console.log("I'm trying to delete VPS with id = [" + vpsId + "]");

                    /* Remove the VPS from the APS controller DB */
                    when(vpsStore.remove(vpsId),
                        /* If success, process the next VPS until the list is empty */
                        function(){
                            console.log("VPS with id = [" + vpsId + "] removed");
                            selectionArray.splice(selectionArray.indexOf(vpsId),1);
                            vpsGrid.refresh();
                            refresh();
                            if (--counter === 0) { btn.cancel(); }
                        },
                        /* If failure, call the error handler */
                        function(e){
                            displayError(e);
                            if (--counter === 0) { btn.cancel(); }
                        }
                    );
                });
            };
            //_FUNCTIONS_NOT_MODIFIED_END

            /* Define and return widgets */
            return [
                ["aps/Panel", {
                    title: "Virtual Servers"
                },[
                    ["aps/Grid", {
                        id:                this.genId("servers_list"),
                        store:             vpsStore,
                        selectionMode:     "multiple",
                        apsResourceViewId: "server-edit",
                        noDataText: "No servers provisioned",
                        noEntriesFoundText: "No servers meet your search criteria",
                        columns: [{
                            field: "name",
                            name: "Name",
                            type: "resourceName",
                            filter: true
                        }, {
                            field: "hardware.memory",
                            name: "RAM",
                            title: "RAM",
                            type: "integer",
                            filter: true
                        }, {
                            field: "hardware.diskspace",
                            name: "Disk Space",
                            title: "Disk",
                            type: "integer",
                            filter: true
                        }, {
                            field: "hardware.CPU.number",
                            name: "CPU",
                            type: "integer",
                            filter: true
                        }, {
                            field: "platform.OS.name",
                            name: "OS",
                            filter: true
                        }, {
                            field: "state",
                            name: "State",
                            filter: true
                        }
                        ]}, [
                        ["aps/Toolbar", [
                            ["aps/ToolbarButton", {
                                id: this.genId("srv_new"),
                                iconClass:"fa-plus",
                                type: "primary",
                                label: "Add New VPS",
                                onClick: add
                            }],
                            ["aps/ToolbarButton", {
                                id: this.genId("srv_start"),
                                iconClass:"fa-play",
                                type: "success",
                                label: "Start",
                                requireItems: true,
                                onClick: start
                            }],
                            ["aps/ToolbarButton", {
                                id: this.genId("srv_stop"),
                                iconClass:"fa-pause",
                                type: "warning",
                                label: "Stop",
                                requireItems: true,
                                onClick: stop
                            }],
                            ["aps/ToolbarButton", {
                                id: this.genId("srv_delete"),
                                iconClass:"fa-trash",
                                type: "danger",
                                label: "Delete",
                                requireItems: true,
                                onClick: remove
                            }],
                            ["aps/ToolbarButton", {
                                id: this.genId("srv_refresh"),
                                iconClass:"fa-refresh",
                                label: "Refresh",
                                autoBusy: false,
                                onClick: refresh
                            }]
                        ]]
                    ]]
                ]],
                ["aps/Panel", {
                    title: "Vendor Rated Charge Reports"
                },[
                    ["aps/Grid", {
                        id: this.genId("vrd_charges"),
                        store: chargeReportCache,
                        noDataText: "No rated data yet",
                        noEntriesFoundText: "No rated data collected yet from the vendor",
                        onRowClick: function(row) {
                            aps.apsc.showPopup({
                                viewId: "http://aps-standard.org/samples/vrd#rated-data",
                                params: {"report": JSON.stringify(row)}
                            });
                        },
                        columns: [{
                            field: "lastInvoiceDate",
                            name: "Report Date-Time, UTC   -- click on a row to get details",
                            filter: true
                        }, {
                            name: "Total for Customer, USD",
                            //filter: true,
                            renderCell: function (row) {
                                var total = 0;
                                row.charges.forEach(function (charge) {
                                    if(charge["accountId"] == accountId)
                                        total = total + charge["totalCost"];
                                });
                                return total.toFixed(2);
                            }
                        }, {
                            name: "Total for Reseller, USD",
                            //filter: true,
                            renderCell: function (row) {
                                var total = 0;
                                row.charges.forEach(function (charge) {
                                    if(charge["accountId"] == parentId)
                                        total = total + charge["totalCost"];
                                });
                                return total.toFixed(2);
                            }

                        }]}, [
                        ["aps/Toolbar", [
                            ["aps/ToolbarButton", {
                                id: this.genId("srv_pollCharge"),
                                iconClass:"fa-refresh",
                                label: "Poll charges",
                                autoBusy: false,
                                onClick: pollCharges
                            }]
                        ]]]
                    ]
                ]]

            ];
        },	// End of Init

        onContext: function(context) {
            lastPollTime = getCurrentTime();
            vpsGrid = this.byId("servers_list");
            ratedDataGrid = this.byId("vrd_charges");
            page = this.byId("apsPageContainer");
            selectionArray = vpsGrid.get("selectionArray");
            var mgmtContext = context.vars.context;
            var account = context.vars.account;
            accountId = account.aps.id;
            parentId = account.parent.aps.id;
            contextId = mgmtContext.aps.id;
            vpsGrid.refresh();
            ratedDataGrid.refresh();
            aps.apsc.hideLoading();
        },

        onHide: function() {
            this.byId("srv_new").cancel();
        }

    });
});
