define([
	"dojo/_base/declare",
	"dojox/mvc/getStateful",
	"dojox/mvc/at",

	"aps/_View",
	"aps/Memory",

	"dojo/text!./../newvps.json"
], function(
	declare,
	getStateful,
	at,

	_View,
	Memory,

	newVPS
) {
	return declare(_View, {
		init: function() {

			/* Declare the data source */
			var oses = new Memory({
				idProperty: "value",
				data: [
					{ value: "centos6",      label: "CentOS 6" },
					{ value: "debian",       label: "Debian" },
					{ value: "windows2008",  label: "Windows 2008 Server" },
					{ value: "windows2012",  label: "Windows 2012 Server" }
				]
			});

			this.model = getStateful({"newVPS": JSON.parse(newVPS)});

			/* Create widgets */
			return [
				["aps/Panel", {id: this.genId("wizardNew_panel1"), title: _("Server")}, [
					["aps/FieldSet", {id: this.genId("srvNew1_")}, [
						["aps/TextBox", {
							id: this.genId("srvNew1_serverName"),
							label: _("Name"),
							gridSize: "md-3 xs-12",
							value: at(this.model.newVPS, "name"),
							required: true
						}],
						["aps/TextBox", {
							id: this.genId("srvNew1_description"),
							label: _("Description"),
							gridSize: "md-3 xs-12",
							value: at(this.model.newVPS, "description")
						}]
					]]
				]],
				["aps/Panel", { id: this.genId("wizardNew_panel2"), title: _("Configuration")}, [
					["aps/FieldSet", {id: this.genId("srvNew2_")}, [
						["aps/Select", {
							id: this.genId("srvNew1_os"),
							gridSize: 'md-3 xs-12',
							label: _( "OS"),
							value: at(this.model.newVPS.platform.OS, "name"),
							store: oses
						}],
						["aps/Spinner", {
							id: this.genId("srvNew1_cpu"),
							gridSize: 'md-3 xs-12',
							label: _( "CPU Number"),
							minimum: 1, maximum: 16,
							value: at(this.model.newVPS.hardware.CPU, "number"),
							step: 1
						}],
						["aps/Spinner", {
							id: this.genId("srvNew1_disk"),
							gridSize: 'md-3 xs-12',
							label: _("Disk Space"),
							minimum: 1,
							maximum: 100,
							value: at(this.model.newVPS.hardware, "diskspace"),
							legend: _("GB"),
							step: 1
						}],
						["aps/Spinner", {
							id: this.genId("srvNew1_ram"),
							gridSize: 'md-3 xs-12',
							label: _("RAM"),
							minimum: 128,
							maximum: 8192,
							value: at(this.model.newVPS.hardware, "memory"),
							step: 128,
							legend: _("MB")
						}],
						["aps/CheckBox", {
							id: this.genId("srvNew1_srvOsKernel"),
							description: _('I will manage the OS Kernel'),
							checked: at(this.model.newVPS.hardware, "VM")
						}]
					]]
				]]
			];
		},	// End of Init

		onNext: function() {
			/* Validate the values assigned to widgets... */
			if (!this.validate()) {
				aps.apsc.cancelProcessing();
				return;
			}
			/* ...and proceed to the next screen */
			aps.apsc.next(this.model.newVPS);
		},	// End of the Next call-back

		onShow: function() {
			if (!aps.context.wizardData["http://aps-standard.org/samples/vrd#vps-props"]) {
				this.model.set("newVPS", JSON.parse(newVPS));
			}
		}
	});	// End of Declare
}); // End of Define
